import { Badge } from '@mui/material'
import { Event, verifyEvent } from 'nostr-tools'
import { useState, useEffect } from 'react'
import { Meta, ProfileMetadata } from '../../types'
import { hexToNpub, parseJson } from '../../utils'
import styles from './style.module.scss'
import { UserAvatar } from '../UserAvatar'
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'
import { faCheck, faExclamation } from '@fortawesome/free-solid-svg-icons'

enum SignStatus {
  Signed = 'Signed',
  Pending = 'Pending',
  Invalid = 'Invalid Sign'
}

type DisplaySignerProps = {
  meta: Meta
  profile: ProfileMetadata
  pubkey: string
}

export const DisplaySigner = ({
  meta,
  profile,
  pubkey
}: DisplaySignerProps) => {
  const [signStatus, setSignedStatus] = useState<SignStatus>()

  useEffect(() => {
    if (!meta) return

    const updateSignStatus = async () => {
      const npub = hexToNpub(pubkey)
      if (npub in meta.docSignatures) {
        parseJson<Event>(meta.docSignatures[npub])
          .then((event) => {
            const isValidSignature = verifyEvent(event)
            if (isValidSignature) {
              setSignedStatus(SignStatus.Signed)
            } else {
              setSignedStatus(SignStatus.Invalid)
            }
          })
          .catch((err) => {
            console.log(`err in parsing the docSignatures for ${npub}:>> `, err)
            setSignedStatus(SignStatus.Invalid)
          })
      } else {
        setSignedStatus(SignStatus.Pending)
      }
    }

    updateSignStatus()
  }, [meta, pubkey])

  return (
    <Badge
      className={styles.signer}
      overlap="circular"
      anchorOrigin={{ vertical: 'top', horizontal: 'right' }}
      badgeContent={
        signStatus !== SignStatus.Pending && (
          <div className={styles.statusBadge}>
            {signStatus === SignStatus.Signed && (
              <FontAwesomeIcon icon={faCheck} />
            )}
            {signStatus === SignStatus.Invalid && (
              <FontAwesomeIcon icon={faExclamation} />
            )}
          </div>
        )
      }
    >
      <UserAvatar pubkey={pubkey} image={profile?.picture} />
    </Badge>
  )
}
