import { CheckCircle, Cancel } from '@mui/icons-material'
import { Divider, Tooltip } from '@mui/material'
import { verifyEvent } from 'nostr-tools'
import { useSigitProfiles } from '../../hooks/useSigitProfiles'
import { Meta, SignedEventContent } from '../../types'
import {
  extractFileExtensions,
  formatTimestamp,
  fromUnixTimestamp,
  hexToNpub,
  npubToHex,
  shorten
} from '../../utils'
import { UserAvatar } from '../UserAvatar'
import { useSigitMeta } from '../../hooks/useSigitMeta'
import { UserAvatarGroup } from '../UserAvatarGroup'

import styles from './style.module.scss'
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'
import {
  faCalendar,
  faCalendarCheck,
  faCalendarPlus,
  faEye,
  faFile,
  faFileCircleExclamation
} from '@fortawesome/free-solid-svg-icons'
import { getExtensionIconLabel } from '../getExtensionIconLabel'
import { useSelector } from 'react-redux'
import { State } from '../../store/rootReducer'

interface FileUsersProps {
  meta: Meta
}

export const FileUsers = ({ meta }: FileUsersProps) => {
  const { usersPubkey } = useSelector((state: State) => state.auth)
  const {
    submittedBy,
    signers,
    viewers,
    fileHashes,
    sig,
    docSignatures,
    parsedSignatureEvents,
    createdAt,
    signedStatus,
    completedAt
  } = useSigitMeta(meta)
  const profiles = useSigitProfiles([
    ...(submittedBy ? [submittedBy] : []),
    ...signers,
    ...viewers
  ])
  const userCanSign =
    typeof usersPubkey !== 'undefined' &&
    signers.includes(hexToNpub(usersPubkey))

  const ext = extractFileExtensions(Object.keys(fileHashes))

  const getPrevSignersSig = (npub: string) => {
    // if user is first signer then use creator's signature
    if (signers[0] === npub) {
      return sig
    }

    // find the index of signer
    const currentSignerIndex = signers.findIndex((signer) => signer === npub)
    // return null if could not found user in signer's list
    if (currentSignerIndex === -1) return null
    // find prev signer
    const prevSigner = signers[currentSignerIndex - 1]

    // get the signature of prev signer
    try {
      const prevSignersEvent = parsedSignatureEvents[prevSigner]
      return prevSignersEvent.sig
    } catch (error) {
      return null
    }
  }

  const displayUser = (pubkey: string, verifySignature = false) => {
    const profile = profiles[pubkey]

    let isValidSignature = false

    if (verifySignature) {
      const npub = hexToNpub(pubkey)
      const signedEventString = docSignatures[npub]
      if (signedEventString) {
        try {
          const signedEvent = JSON.parse(signedEventString)
          const isVerifiedEvent = verifyEvent(signedEvent)

          if (isVerifiedEvent) {
            // get the actual signature of prev signer
            const prevSignersSig = getPrevSignersSig(npub)

            // get the signature of prev signer from the content of current signers signedEvent

            try {
              const obj: SignedEventContent = JSON.parse(signedEvent.content)
              if (
                obj.prevSig &&
                prevSignersSig &&
                obj.prevSig === prevSignersSig
              ) {
                isValidSignature = true
              }
            } catch (error) {
              isValidSignature = false
            }
          }
        } catch (error) {
          console.error(
            `An error occurred in parsing and verifying the signature event for ${pubkey}`,
            error
          )
        }
      }
    }

    return (
      <>
        <UserAvatar
          pubkey={pubkey}
          name={
            profile?.display_name || profile?.name || shorten(hexToNpub(pubkey))
          }
          image={profile?.picture}
        />

        {verifySignature && (
          <>
            {isValidSignature && (
              <Tooltip title="Valid signature">
                <CheckCircle sx={{ color: 'green' }} />
              </Tooltip>
            )}

            {!isValidSignature && (
              <Tooltip title="Invalid signature">
                <Cancel sx={{ color: 'red' }} />
              </Tooltip>
            )}
          </>
        )}
      </>
    )
  }

  return submittedBy ? (
    <div className={styles.container}>
      <div className={styles.section}>
        <p>Signers</p>
        {displayUser(submittedBy)}
        {submittedBy && signers.length ? (
          <Divider orientation="vertical" flexItem />
        ) : null}
        <UserAvatarGroup max={7}>
          {signers.length > 0 &&
            signers.map((signer) => (
              <span key={signer}>{displayUser(npubToHex(signer)!, true)}</span>
            ))}
          {viewers.length > 0 &&
            viewers.map((viewer) => (
              <span key={viewer}>{displayUser(npubToHex(viewer)!)}</span>
            ))}
        </UserAvatarGroup>
      </div>
      <div className={styles.section}>
        <p>Details</p>

        <Tooltip
          title={'Publication date'}
          placement="top"
          arrow
          disableInteractive
        >
          <span className={styles.detailsItem}>
            <FontAwesomeIcon icon={faCalendarPlus} />{' '}
            {createdAt ? formatTimestamp(createdAt) : <>&mdash;</>}
          </span>
        </Tooltip>

        <Tooltip
          title={'Completion date'}
          placement="top"
          arrow
          disableInteractive
        >
          <span className={styles.detailsItem}>
            <FontAwesomeIcon icon={faCalendarCheck} />{' '}
            {completedAt ? formatTimestamp(completedAt) : <>&mdash;</>}
          </span>
        </Tooltip>

        {/* User signed date */}
        {userCanSign ? (
          <Tooltip
            title={'Your signature date'}
            placement="top"
            arrow
            disableInteractive
          >
            <span className={styles.detailsItem}>
              <FontAwesomeIcon icon={faCalendar} />{' '}
              {hexToNpub(usersPubkey) in parsedSignatureEvents ? (
                parsedSignatureEvents[hexToNpub(usersPubkey)].created_at ? (
                  formatTimestamp(
                    fromUnixTimestamp(
                      parsedSignatureEvents[hexToNpub(usersPubkey)].created_at
                    )
                  )
                ) : (
                  <>&mdash;</>
                )
              ) : (
                <>&mdash;</>
              )}
            </span>
          </Tooltip>
        ) : null}
        <span className={styles.detailsItem}>
          <FontAwesomeIcon icon={faEye} /> {signedStatus}
        </span>
        {ext.length > 0 ? (
          <span className={styles.detailsItem}>
            {ext.length > 1 ? (
              <>
                <FontAwesomeIcon icon={faFile} /> Multiple File Types
              </>
            ) : (
              getExtensionIconLabel(ext[0])
            )}
          </span>
        ) : (
          <>
            <FontAwesomeIcon icon={faFileCircleExclamation} /> &mdash;
          </>
        )}
      </div>
    </div>
  ) : undefined
}
